@@ -32,13 +32,25 @@ import AXKeyCodeConstants
 class BaseAXUIElement(_a11y.AXUIElement):
    '''BaseAXUIElement - Base class for UI elements.
 
-      BaseAXUIElement implements three major things:
+      BaseAXUIElement implements four major things:
       1) Factory class methods for getAppRef and getSystemObject which
          properly instantiate the class.
       2) Generators and methods for finding objects for use in child classes.
       3) __getattribute__ call for invoking actions.
       4) waitFor utility based upon AX notifications.
    '''
+   @classmethod
+   def _getRunningApps(cls):
+      '''Get a list of the running applications'''
+      def runLoopAndExit():
+         AppHelper.stopEventLoop()
+      AppHelper.callLater(1, runLoopAndExit)
+      AppHelper.runConsoleEventLoop()
+      # Get a list of running applications
+      ws = AppKit.NSWorkspace.sharedWorkspace()
+      apps = ws.runningApplications()
+      return apps
+
    @classmethod
    def getAppRefByPid(cls, pid):
       '''getAppRef - Get the top level element for the application specified
@@ -70,19 +82,33 @@ class BaseAXUIElement(_a11y.AXUIElement):
          Wildcards are also allowed.
       '''
       # Refresh the runningApplications list
-      def runLoopAndExit():
-         AppHelper.stopEventLoop()
-      AppHelper.callLater(1, runLoopAndExit)
-      AppHelper.runConsoleEventLoop()
-      # Get a list of running applications
-      ws = AppKit.NSWorkspace.sharedWorkspace()
-      apps = ws.runningApplications()
+      apps = cls._getRunningApps()
       for app in apps:
          if fnmatch.fnmatch(app.localizedName(), name):
             pid = app.processIdentifier()
             return cls.getAppRefByPid(pid)
       raise ValueError('Specified application not found in running apps.')
 
+   @classmethod
+   def getFrontmostApp(cls):
+      '''getFrontmostApp - Get the current frontmost application.
+
+         Raise a ValueError exception if no GUI applications are found. 
+      '''
+      # Refresh the runningApplications list
+      apps = cls._getRunningApps()
+      for app in apps:
+         pid = app.processIdentifier()
+         ref = cls.getAppRefByPid(pid)
+         try:
+            if ref.AXFrontmost:
+               return ref
+         except (_a11y.ErrorUnsupported, _a11y.ErrorCannotComplete):
+            # Some applications do not have an explicit GUI
+            # and so will not have an AXFrontmost attribute
+            pass
+      raise ValueError('No GUI application found.')
+
    @classmethod
    def getSystemObject(cls):
       '''getSystemObject - Get the top level system accessibility object'''
@@ -238,6 +264,14 @@ class BaseAXUIElement(_a11y.AXUIElement):
                       global or app specific
           Returns: None or raise ValueError exception
       '''
+      escapedChrs = {
+                       '\n': AXKeyCodeConstants.RETURN,
+                       '\r': AXKeyCodeConstants.RETURN,
+                       '\t': AXKeyCodeConstants.TAB,
+                    }
+      if keychr in escapedChrs:
+         keychr = escapedChrs[keychr]
+
       self._addKeyToQueue(keychr, modFlags, globally=globally)
       self._postQueuedEvents()
 
@@ -484,10 +518,11 @@ class BaseAXUIElement(_a11y.AXUIElement):
          children = target.AXChildren
       except _a11y.Error:
          return
-      for child in children:
-         yield child
-         for c in self._generateChildrenR(child):
-            yield c
+      if children:
+         for child in children:
+            yield child
+            for c in self._generateChildrenR(child):
+               yield c
 
    def _match(self, **kwargs):
       '''_match - Method which indicates if the object matches specified
